import { jsxs, jsx } from '@emotion/react/jsx-runtime';
import { Text, Box, InputGroup, Input, InputRightElement, Spinner, IconButton, Icon } from '@chakra-ui/react';
import { defineComponent, useDisableCondition } from '@qorebase/app-component';
import React from 'react';
import { TargetLock } from '@feedloop/icon';

const InputComponentLabel = ({ title, name, isRequired, isEmpty, isLabel, }) => {
    return (jsxs(Text, Object.assign({ fontSize: "lg", pb: name === "rating" ? "1" : "2", "data-test": `${name}-input-title` }, { children: [title, !isLabel ||
                (!isRequired && !isEmpty && (jsxs(Text, Object.assign({ display: "inline", fontStyle: "italic" }, { children: [" ", "- Optional"] }))))] })));
};
var InputComponentLabel$1 = InputComponentLabel;

const LocationInput = defineComponent({
    name: "Location Input",
    type: "none",
    icon: "LocationInput",
    group: "input",
    display: "inline",
    hidden: true,
    defaultProps: {
        label: "Location Input",
        hidden: false,
        placeholder: "Placeholder here",
        data: {
            inputID: "location",
            field: { type: "string", options: { format: "text" } },
            validation: {
                required: false,
            },
        },
        defaultValue: "",
    },
    propDefinition: {
        label: { group: "Design", type: "expression", options: {} },
        placeholder: {
            group: "Design",
            type: "expression",
            options: {},
        },
        hidden: { group: "Design", type: "boolean", options: {} },
        data: {
            group: "Design",
            type: "state",
            options: {
                inputID: "location",
                field: { type: "string", options: { format: "text" } },
            },
        },
        defaultValue: {
            group: "Design",
            type: "expression",
            options: {},
        },
    },
    Component: (props) => {
        var _a;
        const title = props.hooks.useEvaluate(props.properties.label);
        const placeholder = props.hooks.useEvaluate(props.properties.placeholder);
        const propsDefaultValue = props.hooks.useEvaluate(props.properties.defaultValue);
        const defaultValue = React.useMemo(() => (!propsDefaultValue ? undefined : propsDefaultValue), [propsDefaultValue]);
        const data = props.hooks.useRuntimeInput(props.properties.data, props.data.page.row, {
            defaultValue,
            validate: (value) => {
                if (typeof value === "undefined") {
                    return { error: "" };
                }
                else if (!value) {
                    return { error: "Required field, Please enter the data" };
                }
            },
        });
        const handleChange = React.useCallback((e) => {
            data.onChange(e.currentTarget.value);
        }, [data.onChange]);
        const [supported, setSupported] = React.useState(true);
        const [locating, setLocating] = React.useState(true);
        const getLocation = () => {
            if (navigator.geolocation) {
                setLocating(true);
                navigator.geolocation.getCurrentPosition((position) => {
                    data.onChange(`${position.coords.latitude},${position.coords.longitude}`);
                    setLocating(false);
                });
            }
            else {
                setLocating(false);
                setSupported(false);
            }
        };
        // @ts-ignore
        const { isDisabled } = useDisableCondition(props.instance);
        const disabledProperties = {
            border: "1px solid rgb(226, 232, 240)",
            pointerEvents: isDisabled ? "none" : "auto",
            background: isDisabled && "#E7E7E7",
            color: isDisabled && "#A2A2A2",
        };
        React.useEffect(() => {
            getLocation();
        }, []);
        return (jsxs(Box, Object.assign({ display: props.properties.hidden ? "none" : undefined }, { children: [jsx(InputComponentLabel$1, { title: title, name: "location", isRequired: (_a = props.properties.data.validation) === null || _a === void 0 ? void 0 : _a.required, isEmpty: title.length === 0 }), jsxs(InputGroup, Object.assign({ size: "md" }, { children: [jsx(Input, { pr: "10", style: disabledProperties, bgColor: "surface.900", placeholder: placeholder, name: props.properties.data.inputID, defaultValue: data.value, onChange: handleChange, disabled: supported, _disabled: { border: "1px solid rgb(226, 232, 240)" }, "data-test": "location-input" }, props.properties.data.inputID), jsx(InputRightElement, Object.assign({ width: "10" }, { children: locating ? (jsx(Spinner, {})) : (jsx(IconButton, { "aria-label": "locate", variant: "link", icon: jsx(Icon, { as: TargetLock }), onClick: getLocation, "data-test": "location-input-toggle", style: disabledProperties })) }))] })), !!data.validation.error && (jsx(Text, Object.assign({ fontSize: "sm", color: "red", "data-test": "location-input-error" }, { children: data.validation.error })))] })));
    },
});

export { LocationInput as default };
